@@ -18,6 +18,7 @@ use fields (
 use Net::SIP::Debug;
 use Net::SIP::Endpoint::Context;
 use Net::SIP::Util qw(invoke_callback);
+use Scalar::Util 'weaken';
 
 ############################################################################
 # create a new endpoint
@@ -34,7 +35,9 @@ sub new {
 
 	# announce myself as upper layer for incoming packets to
 	# the dispatcher
-	$dispatcher->set_receiver( $self );
+	my $cb = [ \&receive,$self ];
+	weaken( $cb->[1] );
+	$dispatcher->set_receiver( $cb );
 
 	return $self;
 }
@@ -225,7 +228,7 @@ sub close_context {
 # Returns: NONE
 ############################################################################
 sub receive {
-	my Net::SIP::Endpoint $self = shift;
+	my Net::SIP::Endpoint $self = shift || return;
 	my ($packet,$leg,$from) = @_;
 	return $packet->is_response
 		? $self->receive_response( $packet,$leg,$from )
@@ -326,6 +329,13 @@ sub new_response {
 	my ($ctx,$response,$leg,$addr) = @_;
 
 	$self->{ctx}{ $ctx->callid } = $ctx if $ctx; # keep context
+	if ( $ctx && ! $response->get_header( 'contact' ) && $response->method eq 'INVITE' ) {
+		my $code = $response->code;
+		if ( $code>=200 && $code<300 ) {
+			# 2xx response requires contact header
+			$response->set_header( contact => $ctx->contact );
+		}
+	}
 	$self->{dispatcher}->deliver( $response,
 		leg      => $leg,
 		dst_addr => $addr,
