@@ -1,4 +1,4 @@
-###########################################################################
+#########################################################################
 # Net::SIP::Simple
 # simple methods for creation of UAC,UAS
 # - register    register Address
@@ -23,6 +23,8 @@ use fields (
 	'from',               # SIP address of caller
 	'domain',             # default domain for SIP addresses
 	'last_error',         # last error
+	'options',            # hash with field,values for response to OPTIONS request
+	'ua_cleanup',         # cleanup callbacks
 );
 
 use Carp qw(croak);
@@ -57,6 +59,7 @@ use Net::SIP::Debug;
 #     registrar      - use registrar for registration
 #     auth           - auth data: see Request->authorize for format
 #     from           - myself, used for calls and registration
+#     options        - hash with fields,values for reply to OPTIONS request
 #     loop           - predefined Net::SIP::Dispatcher::Eventloop, used if
 #                      shared between UAs
 #     dispatcher     - predefined Net::SIP::Dispatcher, used if
@@ -86,6 +89,25 @@ sub new {
 			if $from !~m{\s} && $from !~m{\@};
 	}
 
+	my $ua_cleanup = [];
+	my $self = fields::new( $class );
+
+	my $options = delete $args{options} || {};
+	{
+		@{$options}{ map { lc } keys(%$options) } = values(%$options); # lc keys
+		my %default_options = (
+			allow => 'INVITE, ACK, CANCEL, OPTIONS, BYE',
+			accept => 'application/sdp',
+			'accept-encoding' => '',
+			'accept-language' => 'en',
+			supported => '',
+		);
+		while ( my ($k,$v) = each %default_options ) {
+			$options->{$k} = $v if ! defined $options->{$k};
+		}
+	}
+
+
 	my $legs = delete $args{legs} || delete $args{leg};
 	$legs = [ $legs ] if $legs && ref($legs) ne 'ARRAY';
 	$legs ||= [];
@@ -134,10 +156,16 @@ sub new {
 			domain2proxy => $d2p,
 		);
 	}
+	push @$ua_cleanup, [ 
+		sub {
+			my ($self,$legs) = @_;
+			$self->{dispatcher}->remove_leg(@$legs);
+		}, 
+		$self,$legs
+	] if @$legs;
 
 	my $endpoint = Net::SIP::Endpoint->new( $disp );
 
-	my $self = fields::new( $class );
 	my $routes = delete $args{routes} || delete $args{route};
 	%$self = (
 		auth => $auth,
@@ -148,10 +176,25 @@ sub new {
 		dispatcher => $disp,
 		loop => $loop,
 		route => $routes,
+		options => $options,
+		ua_cleanup => $ua_cleanup,
 	);
 	return $self;
 }
 
+###########################################################################
+# cleanup object, e.g. remove legs it added to dispatcher
+# Args: ($self)
+# Returns: NONE
+###########################################################################
+sub cleanup {
+	my Net::SIP::Simple $self = shift;
+	while ( my $cb = shift @{ $self->{ua_cleanup} } ) {
+		invoke_callback($cb,$self)
+	}
+	%$self = ();
+}
+
 ###########################################################################
 # get last error or set it
 # Args: ($self;$err)
@@ -348,11 +391,17 @@ sub listen {
 	# handle new requests
 	my $receive = sub {
 		my ($self,$args,$endpoint,$ctx,$request,$leg,$from) = @_;
-		$request->method eq 'INVITE' or do {
+		my $method = $request->method;
+		if ( $method eq 'OPTIONS' ) {
+			my $response = $request->create_response( '200','OK',$self->{options} );
+			$self->{endpoint}->new_response( $ctx,$response,$leg,$from );
+			$self->{endpoint}->close_context( $ctx );
+			return;
+		} elsif ( $method ne 'INVITE' ) {
 			DEBUG( 10,"drop non-INVITE request: ".$request->dump );
 			$self->{endpoint}->close_context( $ctx );
 			return;
-		};
+		}
 
 		if ( my $filter = $args->{filter} ) {
 			my $rv = invoke_callback( $filter, $ctx->{from},$request );
