@@ -66,6 +66,7 @@ sub new {
 	my ($class,$control,$ctx,$param) = @_;
 	my $self = fields::new( $class );
 	%$self = %$control;
+	$self->{ua_cleanup} = [];
 	$self->{ctx} = ref($ctx) ? $ctx : {
 		to => $ctx,
 		from => $self->{from},
@@ -91,8 +92,11 @@ sub cleanup {
 	while ( my $cb = shift @{ $self->{call_cleanup} } ) {
 		invoke_callback($cb,$self)
 	}
-	%$self = ();
-	DEBUG( 100,"done" );
+	if ( my $ctx = $self->{ctx} ) {
+		$self->{endpoint}->close_context( $ctx );
+	}
+	$self->{param} = {};
+	$self->SUPER::cleanup;
 }
 
 sub rtp_cleanup {
